#!/usr/bin/env python
# encoding: utf-8
"""Helps with command line commands."""

import os
import sys
import json
import click
import yaml
import loaih
import loaih.build

@click.group()
def cli():
    """Helps with command line commands."""

@cli.command()
@click.option('-j', '--json', 'jsonout', default=False, is_flag=True, help="Output format in json.")
@click.option('--default-to-current', '-d', is_flag=True, default=False, help="If no versions are found, default to current one (for daily builds). Default: do not default to current.")
@click.argument('query')
def getversion(query, jsonout, default_to_current):
    """Get download information for named or numbered versions."""

    batchlist = []
    queries = []
    if ',' in query:
        queries.extend(query.split(','))
    else:
        queries.append(query)

    for singlequery in queries:
        elem = loaih.Solver.parse(singlequery, default_to_current)
        if elem.version not in { None, "" }:
            batchlist.append(elem)

    if len(batchlist) > 0:
        if jsonout:
            click.echo(json.dumps([x.to_dict() for x in batchlist ]))
        else:
            for value in batchlist:
                click.echo(value)

@cli.command()
@click.option("--verbose", '-v', is_flag=True, default=False, help="Show building phases.", show_default=True)
@click.argument("yamlfile")
def batch(yamlfile, verbose):
    """Builds a collection of AppImages based on YAML file."""
    # Defaults for a batch building is definitely more different than a manual
    # one. To reflect this behaviour, I decided to split the commands between
    # batch (bulk creation) and build (manual building).

    # Check if yamlfile exists.
    if not os.path.exists(os.path.abspath(yamlfile)):
        click.echo(f"YAML file {yamlfile} does not exists or is unreadable.")
        sys.exit(1)

    # This is a buildfile. So we have to load the file and pass the build options ourselves.
    config = {}
    with open(os.path.abspath(yamlfile), 'r', encoding= 'utf-8') as file:
        config = yaml.safe_load(file)

    # With the config file, we ignore all the command line options and set
    # generic default.
    for cbuild in config['builds']:
        # Loop a run for each build.
        collection = loaih.build.Collection(cbuild['query'])

        for obj in collection:
            # Configuration phase
            obj.verbose = verbose
            obj.language = cbuild['language']
            obj.offline_help = cbuild['offline_help']
            obj.portable = cbuild['portable']
            obj.updatable = True
            obj.storage_path = "/srv/http/appimage.sys42.eu"
            if 'repo' in config['data'] and config['data']['repo']:
                obj.storage_path = config['data']['repo']
            obj.download_path = "/var/tmp/downloads"
            if 'download' in config['data'] and config['data']['download']:
                obj.download_path = config['data']['download']
            if 'http' in obj.storage_path:
                obj.remoterepo = True
                obj.remote_host = "ciccio.libreitalia.org"
                if 'remote_host' in config['data'] and config['data']['remote_host']:
                    obj.remote_host = config['data']['remote_host']
                obj.remote_path = "/var/lib/nethserver/vhost/appimages"
                if 'remote_path' in config['data'] and config['data']['remote_path']:
                    obj.remote_path = config['data']['remote_path']

            if 'sign' in config['data'] and config['data']['sign']:
                obj.sign = True

            # Build phase
            obj.calculate()
            obj.check()
            obj.download()
            obj.build()
            obj.checksums()
            if obj.remoterepo and obj.appnamedir:
                obj.generalize_and_link(obj.appnamedir)
            obj.publish()
            if not obj.remoterepo:
                obj.generalize_and_link()
            del obj


@cli.command()
@click.option('-a', '--arch', 'arch', default='x86_64',
    type=click.Choice(['x86', 'x86_64', 'all'], case_sensitive=False), help="Build the AppImage for a specific architecture. If there is no specific options, the process will build for both architectures (if available). Default: x86_64")
@click.option('--check', '-c', is_flag=True, default=False,
    help="Checks in the repository path if the queried version is existent. Default: do not check")
@click.option('-d', '--download-path', 'download_path', default = '/var/tmp/downloads', type=str, help="Path to the download folder. Default: /var/tmp/downloads")
@click.option('-l', '--language', 'language', default = 'basic', type=str, help="Languages to be included. Options: basic, standard, full, a language string (e.g. 'it') or a list of languages comma separated (e.g.: 'en-US,en-GB,it'). Default: basic")
@click.option('-o/-O', '--offline-help/--no-offline-help', 'offline', default = False, help="Include or not the offline help for the chosen languages. Default: no offline help")
@click.option('-p/-P', '--portable/--no-portable', 'portable', default = False,
    help="Create a portable version of the AppImage or not. Default: no portable")
@click.option('-r', '--repo-path', 'repo_path', default = '.', type=str, help="Path to the final storage of the AppImage. Default: current directory")
@click.option('-s/-S', '--sign/--no-sign', 'sign', default=True, help="Wether to sign the build. Default: sign")
@click.option('-u/-U', '--updatable/--no-updatable', 'updatable', default = True, help="Create an updatable version of the AppImage or not. Default: updatable")
@click.argument('query')
def build(arch, language, offline, portable, updatable, download_path, repo_path, check, sign, query):
    """Builds an Appimage with the provided options."""

    # Multiple query support
    queries = []
    if ',' in query:
        queries.extend(query.split(','))
    else:
        queries.append(query)

    # Parsing options
    arches = []
    if arch.lower() == 'all':
        # We need to build it twice.
        arches = [ 'x86', 'x86_64' ]
    else:
        arches = [ arch.lower() ]

    for q in queries:
        collection = loaih.build.Collection(q, arches)
        for appbuild in collection:
            # Configuration phase
            appbuild.tidy_folder = False
            appbuild.language = language
            appbuild.offline_help = offline
            appbuild.portable = portable
            appbuild.updatable = updatable
            if repo_path == '.':
                repo_path = os.getcwd()
            appbuild.storage_path = repo_path
            appbuild.download_path = download_path

            if sign:
                appbuild.sign = True

            # Running phase
            appbuild.calculate()

            if check:
                appbuild.check()

            appbuild.download()
            appbuild.appbuild()
            appbuild.checksums()
            appbuild.publish()
            appbuild.generalize_and_link()
            del appbuild
